function [] = calculateGpsToas()
%CALCULATEGPSTOAS Calculates the SAT TOAs of all soundsources.
%   Calculates the SAT TOAs by artoa.toa.predictFromGps. Adds the
%   soundsource offset and drift if already selected.

global artoaWorkspace artoaDataInput artoaConfig;

%% Get all soundsources
fnames = fieldnames(artoaWorkspace.filteredSoundsources);

artoaWorkspace.editTimeOfArrival.satToa = struct();


%% Calculate GPS TOAs for every soundsource
for o = 1:length(fnames)
    % initialize data storage
    artoaWorkspace.editTimeOfArrival.satToa.(fnames{o}) = struct();
    % calculate rafos dates of soundsource
    soundSourceBegin = artoa.convert.dmy2rd(artoaDataInput.soundsources.(fnames{o}).begemis(3), ...
        artoaDataInput.soundsources.(fnames{o}).begemis(2), ...
        artoaDataInput.soundsources.(fnames{o}).begemis(1));
    soundSourceEnd = artoa.convert.dmy2rd(artoaDataInput.soundsources.(fnames{o}).endemis(3), ...
        artoaDataInput.soundsources.(fnames{o}).endemis(2), ...
        artoaDataInput.soundsources.(fnames{o}).endemis(1));
    
    if ~artoa.soundsources.hasAppliedToa(fnames{o}, artoaWorkspace.toaData)     
        if artoa.data.hasMember(artoaWorkspace, {'trackParameter', 'soundspeedMethodString'})
            method = artoaWorkspace.trackParameter.soundspeedMethodString;
        else
            method = 'del grosso';
        end

        soundvelocity = artoa.data.calculateSoundVelocity( ...
            artoaWorkspace.temperature(artoaWorkspace.statusTemperature == 1), ...
            artoaWorkspace.pressure(artoaWorkspace.statusPressure == 1), ...
            method ...
        );
    elseif any(strcmp(artoaWorkspace.editOffsets.offsets.Properties.RowNames, fnames(o)))
        soundvelocity = artoaWorkspace.editOffsets.offsets{fnames{o}, 'AppliedSoundspeed'};
    else
        soundvelocity = NaN;
    end
    

    [gpsDates, predictedToas, launchDateToa] = artoa.toa.predictFromGps( ...
        artoaDataInput.rfb, artoaWorkspace.filteredSoundsources.(fnames{o}), ...
        soundvelocity, ...
        artoa.data.getMember(artoaConfig, {'leapseconds'}) ...
    );

    % add soundsource drift
    if artoa.data.hasMember(artoaWorkspace, {'editOffsets', 'offsets'}) ...
            && any(contains(artoaWorkspace.editOffsets.offsets.Properties.RowNames, fnames{o})) ...
            && artoaWorkspace.editOffsets.useOffsets
        offset = artoaWorkspace.editOffsets.offsets{fnames{o}, 'AppliedOffset'};
        drift = artoaWorkspace.editOffsets.offsets{fnames{o}, 'AppliedDrift'};
        % check if Inf
        if isinf(offset)
            offset = 0;
        end
        if isinf(drift)
            drift = 0;
        end
        % no sorting required, because fourth parameter contains timesteps
        predictedToas = artoa.toa.addDrift( ...
            predictedToas, ...
            offset, ...
            drift, ...
            floor(gpsDates - soundSourceBegin) ...
        );
        % add offset and drift to launch toa
        launchDateToa(2) = artoa.toa.addDrift( ...
            launchDateToa(2), ...
            offset, ...
            drift ...
        );
    end

    % remove gps dates that are past end mission of sound source
    selection = (gpsDates > soundSourceEnd);
    gpsDates(selection) = NaN;
    predictedToas(selection) = NaN;
    % remove gps dates before deployment of soundsource
    selection = (gpsDates < soundSourceBegin);
    gpsDates(selection) = NaN;
    predictedToas(selection) = NaN;
    clear selection;
    
    % save to workspace
    artoaWorkspace.editTimeOfArrival.satToa.(fnames{o}).gpsDate = gpsDates;
    artoaWorkspace.editTimeOfArrival.satToa.(fnames{o}).gpsToa = predictedToas;
    artoaWorkspace.editTimeOfArrival.satToa.(fnames{o}).gpsLaunchDateToa = launchDateToa;
end


end

