%function [A, B, X, unfilteredA, unfilteredB, D] = solve(pRfb, pSoundsources, pTrackingParameter, pToaData, pSatData, pLeapsecondsMatrix, pOffsetsParameter)
% Olaf Boebel 2022-11-03   added 'soundspeed' to output arguments
function [A, B, X, unfilteredA, unfilteredB, D, soundspeed] = solve(pRfb, pSoundsources, pTrackingParameter, pToaData, pSatData, pLeapsecondsMatrix, pOffsetsParameter)


%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

%% Construct matrices A, B and C
[unfilteredA, unfilteredB, unfilteredC, D] = artoa.offsets.createCalculationTables( ...
    pRfb, ...
    pSoundsources, ...
    pTrackingParameter, ...
    pOffsetsParameter, ...
    pToaData, ...
    pSatData, ...
    pOffsetsParameter.offsets(:, 'AppliedSoundspeed'), ...
    pLeapsecondsMatrix ...
);

%OB% 20221018  The above function call passes the Sosos' AppliedSoundspeed to artoa.offsets.createCalculationTables
%If it is NaN, solve does not work, unclear if tables are %constructed nevertheless.  Why should the Soso's sound speed be mandatory
%and not a fitting parameter?

%% Subtract all known offsets
[B] = artoa.offsets.subtractKnownOffsets( ...
    unfilteredA, ...
    unfilteredB, ...
    pOffsetsParameter.offsets ...
);

%% Subtract all known drifts
[B] = artoa.offsets.subtractKnownDrifts( ...
    unfilteredA, ...
    B, ...
    pOffsetsParameter.offsets ...
);

%% Optimize matrices for calculation
[A, unfilteredA] = artoa.offsets.filterCalculationTables( ...
    unfilteredA, ...
    pOffsetsParameter.offsets ...
    );

%% Subtract window start and soundsource reference time
B.toa = B.toa - unfilteredC.ReferenceTime;

%% Use soundspeed
if ~isnan(pOffsetsParameter.offsets{'Float', 'AppliedSoundspeed'})
    if isempty(A)
        return; % nothing to be done, everything is given and soundspeed is fixed
    end
    
    % Distances are in km, soundspeed m/s
    B.toa = B.toa - A{:, 'Distances'} / (pOffsetsParameter.offsets{'Float', 'AppliedSoundspeed'} / 1000);
    A.Distances = [];
end

%% Solve it!
X = A{:, :} \ B.toa;

% Olaf Boebel 2022 11 03
% the above solves the liner equation system if solvable.  However, due to
% noise this is not necessarily the case.   Rather, we seek to find a
% solution which minimizes the differnce between the found solution and the
% true solution. This is given by: 

H = A{:, :};
B2 = H'*B.toa;
A2 = H'*H;
X2 = A2\B2;
DET = det(A2)
CONDITION = rcond(A2)
[X X2]

X = X2;  

% End Olaf Boebel 2022 11 03 

%% Apply row names to X
if isrow(X)
    X = X';
end
X = table(X, 'VariableNames', {'Value'}, 'RowNames', A.Properties.VariableNames);
% Rename Distance column to Soundspeed
varNames = X.Properties.RowNames;
for i = 1:length(varNames)
    if strcmp(varNames{i}, 'Distances')
        X.Properties.RowNames{i} = 'Soundspeed';
        X{'Soundspeed', 'Value'} = 1 / X{'Soundspeed', 'Value'};
    end
end

%OB% 20221007
soundspeed = pOffsetsParameter.offsets{'Float', 'AppliedSoundspeed'};   

end

