function [S_all,S,SBD] = process_moms(filepath, imei)

% Find and indentify mobile originated messages (mom) from Iridium SBD
% Transmitter. 
%
%
% ATTENTION: Include path to repository /oze/functions !

% Optimare Sensorsysteme, 21.10.2010 M.Busack

% compatibility fix, LP, 26.04.17:
S_all = struct();

% Get version from SVN
S.VERSION = '$Id: SBD2PROF.m 999 2012-06-18 10:00:00Z michael $';
%S_all.empty=[];
%clear all;

% % Set filepath or get path via file dialog 
% filepath='/home/michael/src/matlab/sbd2prof/sbd';
% filepath=uigetdir;

% Added support for searching specific imei
% Get all files from directory with .sbd extension
if (~exist('imei', 'var'))
    D = dir(fullfile(filepath,'*.sbd'));
else
    D = dir(fullfile(filepath, ['*' char(imei) '*.sbd']));
end

filelist=char(D.name);

% Create figure
%figure;
%hold on;

% bugfix: Output argument "SBD" (and maybe others) not assigned during
% call to "nemo.optimare.process_moms".
% LP, 27.04.17
if (length(D) == 0)
    SBD = [];
end

% Read files
for i=1:length(D)
    % Open file
    infile=fullfile(filepath,filelist(i,:));
    % Read Message
    SBD(i) = nemo.optimare.readSBDmsg(infile);             
end


% Init processing variables
state = 'Readpart';    
findex = 0;         % File index
grpflag = 0;        % Message group flag, set to 1 if startmsg was found. Set to 0 at end of group.
msgcnt = 1;         % Message counter
ofid = 0;           % No output file
exit = 0;

% Init overall energy variables
pnr = 0;
E.ENERGYCOMPLETE = 0;

% Process messages
while exit == 0
    
    %state
    switch state
        
        case 'Readpart'
            % Increment file counter index
            findex = findex + 1;
            if findex <= length(D)
                % We have at least one file to read.
                % Read SBD message
                filename = fullfile(filepath,filelist(findex,:));
                %SBD(findex) = nemo.optimare.readSBDmsg(filename);
                % Check if we have a valid message type
                if SBD(findex).status == 1
                    % Yes, this is a known device/data message
                    state = 'Getpart';
                else
                    % No, try again with next message
                    disp('Error: File has unknown device/data type');
                    state = 'Readpart';
                end
            else
                % We reached the end of filelist.
                % Before end, check if there is a group pending.
                if grpflag == 1
                    % A message group is pending, finish this first.
                    state = 'Endgroup';
                else
                    % Everything if finished, end function.
                    exit = 1;
                end
            end
            
        case 'Getpart'
            % Decodable message was found
            if SBD(findex).elementary_num == 1
                % This is the first message of a group
                if grpflag == 0
                    % Last group was finished successfully.
                    % => Everything is ok.
                    state = 'Startgroup';
                else
                    % We have an incomplete group pending.
                    % => Finish this group before starting the next one.
                    % Because findex in incremented after process,
                    % decrement here to prevent skipping of a file.
                    findex = findex - 1; 
                    state = 'Endgroup';
                end
            else
                % This is not the first message of a group
                if grpflag == 0
                    % Group was not started before.
                    % => This can happen if the first message is missing
                    state = 'Startgroup';
                else
                    % Start of group was received before. 
                    % => Everything is ok.
                    % Exception: Last message of group before is missing
                    % AND first message of next group is missing also !
                    % TBD!!
                    state = 'Checkpart';
                end
            end
                
        case 'Startgroup'
            % Start of message group found.
            % First, clear data from group before.
            clear S;
            grpflag = 1;
            % Init device/data type
            S.DEVICE = SBD(findex).device;
            S.DATATYPE = SBD(findex).data;
            % Set message counter
            msgcnt = SBD(findex).elementary_num;
            % Init buffer
            l = SBD(findex).total_num * SBD(findex).ldata + 10;
            l = l * 1.3;
            l = ceil(l);              % OB:  added ceil, as ones requires interger parameter
            S.uD=ones(1,l)*NaN;    
            S.sD=S.uD;
            % Offset for uD/sD Array. All messages must have the same
            % datasize here (expect the last one).
            S.DATAPAYLOAD = SBD(findex).ldata;
            % Create output file name with .out extension
            filename = filelist(findex,:);
            l = length(filename);
            filename(l) = 't';
            filename(l-1) = 'u';
            filename(l-2) = 'o';
            % Open file for output
            outfile = fullfile(filepath,filename);
            %ofid = fopen(outfile,'w');
            % Next state
            state = 'Checkpart';

        case 'Checkpart'
            % Check for message consitence
            if SBD(findex).elementary_num == msgcnt
                % Expected message number is ok
                state = 'Writepart';
            else
                % Message number read is not the expected one
                % This can happen if one message/file is missing
                %fprintf(ofid,'\r\nFilename: %s  Device: %s  Data type: %s  Message: %3d of %3d, not found',...
                %    filelist(findex,:), SBD(findex).device, SBD(findex).data, msgcnt, SBD(findex).total_num);
                % What to do ?
                if msgcnt < SBD(findex).elementary_num
                    % Expected message number is smaller than received one.
                    % This can happen if a group file in the middle is missing. 
                    % Try again expecting next message.
                    msgcnt = msgcnt + 1;
                else
                    % Expected message number is larger than received one.
                    state = 'Endgroup';
                    
                    % Ist das richtig ? :
                    % This can happen if this is the next group, already.
                    % Maybe the last message from the group before and the
                    % first message of the next group are missing.
                end
            end
            
        case 'Writepart'
            % Write header information to file
            %fprintf(ofid,'\r\nFilename: %s  Device: %s  Data type: %s  Message: %3d of %3d, %3d bytes of data',...
            %    filelist(findex,:), SBD(findex).device, SBD(findex).data, SBD(findex).elementary_num,...
            %    SBD(findex).total_num, SBD(findex).ldata);
            % Copy data to buffer
            offset = (SBD(findex).elementary_num-1)*S.DATAPAYLOAD;
            i = offset+1:offset+SBD(findex).ldata;
            S.uD(i) = SBD(findex).udata;
            S.sD(i) = SBD(findex).sdata;
            % Check if message is last in set
            if SBD(findex).elementary_num == SBD(findex).total_num
                % Message is last one in set, end of group reached.
                state = 'Endgroup';
            else
                % Not the last message, read next one.
                msgcnt = msgcnt + 1;
                state = 'Readpart';
            end
            
        case 'Endgroup'
            % Finish group
            % Wir auch aufgerufen, wenn die leltzte msg fehlt !
            % Close file.
            %fprintf(ofid,'\r\nNo more messages to read.');
            %fclose(ofid);
            % Set flag to 'group closed successfully'          
            grpflag = 0;
            state = 'Process';
            
        case 'Process'
            try
            % Process the received group
            % First, decode data
            S = nemo.optimare.decode_nemo(S);
            % Write data to output file
            %write_NEMO(S,outfile);
            catch ex
            end
            if findex > length(D)
                exit = 1;
            else
                %state = 'Energycalculation';    % with energy calculation
                state = 'Readpart';             % without any calculations
            end
            expr=['S_all.S_' num2str(findex) '=S;']; 
            eval(expr);
            
        case 'Energycalculation'
            % Perform energy calculation from mission runtime and piston
            % movements
            if isfield(S,'DATA_RECORDS1'),
                if S.DATA_RECORDS1(4) > 0
                    %plot(S.DS4_PRES+findex);     % Datset 4 pressure
                    %plot(S.PROFILE_NUMBER, S.DS4_LICH1+S.DS4_LICH2+S.DS4_LICH3, '.-b');
                    %plot(S.PROFILE_NUMBER,S.DS4_LICH1,'.-r',...
                    %    S.PROFILE_NUMBER,S.DS4_LICH2,'.-g',...
                    %    S.PROFILE_NUMBER,S.DS4_LICH3,'.-b');

                end
            end
            % Calculate overall energy (works only if all data from same float)
            if isfield(S,'PROFILE_NUMBER'),
                pnr = pnr + 1;
                E.VALID(pnr) = 1;
                E.PROFNR(pnr) = S.PROFILE_NUMBER;
                
                if ( (isfield(S,'PIST_SURFACE')) & (isfield(S,'PIST_PARKING')) )
                    E.PISTON_DESCENT(pnr) = S.PIST_PARKING - S.PIST_SURFACE;
                else
                    E.VALID(pnr) = 0;
                end
                if ( (isfield(S,'PIST_DEPTH')) & (isfield(S,'PIST_EOP')) & (E.VALID(pnr) == 1) )
                    E.PISTON_ASCENT(pnr) = S.PIST_DEPTH - S.PIST_EOP;
                else
                    E.VALID(pnr) = 0;
                end
                if ( (isfield(S,'CURR_MOTOR_MEAN')) & (E.VALID(pnr) == 1) )
                    E.MOTORCURRENT(pnr) = S.CURR_MOTOR_MEAN;
                else
                    E.VALID(pnr) = 0;
                end
                if E.VALID(pnr) == 1
                    E.ENERGY(pnr) = (E.PISTON_DESCENT(pnr) + E.PISTON_ASCENT(pnr)) * E.MOTORCURRENT(pnr);
                    E.ENERGYCOMPLETE = E.ENERGYCOMPLETE + E.ENERGY(pnr);
                end
            end
            % Start again
            state = 'Readpart';
            
        otherwise
            % End loop
            disp('State machine error');
            exit = 1;
    end
end
 
%hold off;

%whos;

% 
%        
%                 % This is the first message of a set.
%                 if ( (findex > 1) & (msgcnt < SBD(findex-1).total_num) )
%                     % Only if this is NOT the first file from the list
%                     % and the expected elementray number (msgcnt) is
%                     % smaller than the total number of this set.
%                     % This can happen if the last set was not received 
%                     % completly (one or more messages missing at the
%                     % end)
%                     % Todo: Search for any of the missing files.
%                     state = 'Checkpart';
%                         
%                     % Note: msgcnt is equal to total number if last
%                     % set was completly received. So, we have a set with 
%                     % at least the last file missing.
%                     % Use header informations from last message.
%                     %SBD(findex).device = SBD(findex-1).device;
%                     %SBD(findex).data = SBD(findex-1).data;
%                     %SBD(findex).total_num = SBD(findex-1).total_num;
%                     %msgcnt = msgcnt + 1;
%                     %state = 'Checkpart';
%                         
%                 else
%                     % Start of message set (i.e. a profile) found
%                     if ofid > 0
%                         % Close open output file
%                         % This can happen, if last file of message set is
%                         % missing
%                         fclose(ofid);
%                     end
%                     % Start of new message set
%                     state = 'Startpart';
%                 end
%             else 
%                 % Message is not the first one
%                 if ofid == 0
%                     % No output file was created before.
%                     % This can happen, if first message of set is missing
%                     state = 'Startpart';
%                     msgcnt = SBD(findex).elementary_num;
%                 else
%                     % Output file present, regular writing of data
%                     state = 'Checkpart';
%                 end
%             end



%end