function [ nemo_data ] = create_best_guess( p_nemo_data, p_estimation_method )
%CREATE_BEST_GUESS_POSITION Creates a struct "best_guess_position" in every profile.
%   Calls the following functions to estimate the best position that is
%   available from the given profile data:
%   1) nemo.estimate_best_guess_position
%   2) nemo.estimate_unknown_positions
%   WARNING: Please take care that this is the only possible calling procedure if
%   you want to have consistent position data!
%   
%   Returns: p_nemo_data with an additional struct containing the best
%           position that could be estimated.
%
%   Requirements:
%   - nemo library (this function is part of the library, so you should have it)
%   - Mapping Toolbox
%

%% Initialize return variables
nemo_data = false;

%% Parameter check
parameter_error = false; % indicates if a parameter error was raised

if (ischar(p_nemo_data)) % read profiles using read_profiles.m
    if (~isdir(p_nemo_data)) % parameter is not a directory
        parameter_error = true;
    else % read profiles
        nemo_data = nemo.load.data(p_nemo_data);
    end
else if (~iscell(p_nemo_data)) % raise error
        parameter_error = true;
    else
        nemo_data = p_nemo_data; % store nemo data in return variable
    end
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please provide either a ' ...
        'directory path or a struct created by nemo.load.data!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% create struct and fill with data

% create best_guess_position struct
nemo_data = nemo.postprocessing.position.estimate_best_guess(p_nemo_data);
% estimate all positions that have no gps coordinate
nemo_data = nemo.postprocessing.position.estimate_unknowns(nemo_data, p_estimation_method);

end

